﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using ThinkGeo.MapSuite.Core;

namespace GenerateTiling
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            string version = ThinkGeo.MapSuite.WebEdition.Map.Version; // "MapSuiteCore:3.1.182;WebEdition:3.1.182"
            Map1.MapUnit = GeographyUnit.DecimalDegree;
        }

        protected void btnGenerateTiling_Click(object sender, EventArgs e)
        {
            Collection<ZoomLevel> zoomLevels = GetAllZoomLevels();
            int tileWidth = 256;
            int tileHeight = 256;

            ShapeFileFeatureLayer shapeFileFeatureLayer = new ShapeFileFeatureLayer(@"C:\Program Files\ThinkGeo\Map Suite Desktop Full Edition 3.0\Samples\SampleData\Data\Countries02.shp");
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            MapEngine mapEngine = new MapEngine();
            mapEngine.StaticLayers.Add(shapeFileFeatureLayer);
            GeographyUnit mapUnit = GeographyUnit.DecimalDegree;

            shapeFileFeatureLayer.Open();
            RectangleShape cacheExtent = shapeFileFeatureLayer.GetBoundingBox();
            shapeFileFeatureLayer.Close();

            for (int i = 0; i < zoomLevels.Count; i++)
            {
                MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(zoomLevels[i].Scale, tileWidth, tileHeight, mapUnit);
                BitmapTileCache tileCache = new FileBitmapTileCache(txtCachePath.Text, string.Empty, TileImageFormat.Png, tileMatrix);
                RowColumnRange rowColumnRange = tileCache.TileMatrix.GetIntersectingRowColumnRange(cacheExtent);

                Bitmap bitmap = null;
                RectangleShape tileExtent = tileCache.TileMatrix.GetCell(0, 0).BoundingBox;

                //if (tileExtent.Width > cacheExtent.Width || tileExtent.Height > cacheExtent.Height)
                if (rowColumnRange.MaxColumnIndex == 0 && rowColumnRange.MaxRowIndex == 0)
                {
                    mapEngine.CurrentExtent = tileExtent;
                    bitmap = new Bitmap(tileWidth, tileHeight);
                }
                else
                {
                    int bitMapWidth = tileWidth * ((int)(rowColumnRange.MaxColumnIndex - rowColumnRange.MinColumnIndex) + 1);
                    int bitMapHeight = tileHeight * ((int)(rowColumnRange.MaxRowIndex - rowColumnRange.MinRowIndex) + 1);
                    mapEngine.CurrentExtent = cacheExtent;
                    // if the bitmapWidth is very big, you need to optimize the way generating tiles, otherwise the bitmap will be too big for the memory.
                    // For example, you can always generate a 1024 * 1024 bitmap and then split it into 4 512*512 tiles
                    bitmap = new Bitmap(bitMapWidth, bitMapHeight);
                }

                mapEngine.OpenAllLayers();
                mapEngine.DrawStaticLayers(bitmap, mapUnit);
                mapEngine.DrawDynamicLayers(bitmap, mapUnit);
                mapEngine.CloseAllLayers();

                tileCache.SaveTiles((Bitmap)bitmap.Clone(), tileCache.TileMatrix.BoundingBox);
                bitmap.Dispose();
            }
        }


        private Collection<ZoomLevel> GetAllZoomLevels()
        {
            // We only add the top 5 ZoomLevels here for example. 
            // If you want to add more scales, you need to optimize the way generating tiles otherwise 
            // the bitmap we generated might be too big to be hold in memory.
            GoogleMapZoomLevelSet zoomLevelSet = new GoogleMapZoomLevelSet();
            Collection<ZoomLevel> resultZoomLevels = new Collection<ZoomLevel>();
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel01);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel02);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel03);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel04);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel05);
            return resultZoomLevels;
        }

        protected void btnLoadMap_Click(object sender, EventArgs e)
        {
            Map1.StaticOverlay.ServerCache.CacheDirectory = txtCachePath.Text;
        }
    }
}
